package hclcollector;

#--------------------------------------------------------------------------------------
#  Project	 : hcl
#  File	        : hclcollector
#  Author        : Keerthan Muthurasa <kmuthurasa at mandriva dot com>
#  Created On    : Mon Aug 27 11:15:03 2007
#  Purpose	: Collect hardaware configuration data in order to improve  the 
#  		  compatibility of your system.Data are collected using dmidecode
#  		  tools and lshw tools.Once collected , data are formated in a xml
#  		  format.
# -------------------------------------------------------------------------------------


use File::Temp qw( tempdir );
use File::Basename;
use strict;
use lib qw(/usr/lib/libDrakX);
use common;
use MDK::Common;

require_root_capability();
sub init {	
    my ($dir)=@_;
    my $base=basename($dir);
    rm_rf(glob_("$dir/*"));
    create_lspcidrake_file($dir);
    lshw_collector($dir);
    dmidecode_collector($dir);
    cpuinfo($dir);
    distribution($dir);
    kernel($dir);
    monitor($dir);
    controller($dir);
    input($dir);
    bridge($dir);
    rm_rf("$dir/lspcidrake.out");
    md5file($dir);
    system("cd $dir;tar cfj $base.tar.bz2 *");
}
my $flag=0;
my @pile;
my @indent;


sub dmidecode_collector {
    my ($path)=@_;
    my $start_tag=0;
    my $last_tag;
    my $last_file;
    my $startflag=0;
    my $flagcpt=0;
    my $endflag=0;

    my $pid = open(my $PROC, "-|");
    if ($pid) {
	foreach (<$PROC>) {
	    if (/^$/ || /^Handle/) {
		$start_tag=0;
		if ($last_file ne "" && $last_tag ne "") {
		    while (scalar(@pile) > 0) {
			append_to_file($last_file, pop(@indent) . "</" . pop(@pile) . ">\n");
		    }
		}
		$last_tag="";
		$last_file="";
	    }

	    if (/^BIOS Information/) {
		$start_tag=1;
		$last_tag="bios";
		$last_file="$path/bios.xml"; 
		output($last_file, '') if !-e $last_file;
	    }
	    if (/^System Information/) {
		$start_tag=1;
		$last_tag="system";
		$last_file="$path/system.xml";
		output($last_file, '') if !-e $last_file;
	    }
							
	    if (/^Base Board Information/) {
		$start_tag=1;
		$last_tag="base board";
		$last_file="$path/base_board.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^Chassis Information/) {
		$start_tag=1;
		$last_tag="chassis";
		$last_file="$path/chassis.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^Processor Information/) {
		$start_tag=1;
		$last_tag="processor";
		$last_file="$path/processor.xml";
	    }

	    if (/^Cache Information/) {
		$start_tag=1;
		$last_tag="cache";
		$last_file="$path/cache.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^Memory Module Information/) {
		$start_tag=1;
		$last_tag="memory module";
		$last_file="$path/memory_module.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^Port Connector Information/) {
		$start_tag=1;
		$last_tag="port";
		$last_file="$path/port.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^System Slot Information/) {
		$start_tag=1;
		$last_tag="slot";
		$last_file="$path/slot.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^On Board Device Information/) {
		$start_tag=1;
		$last_tag="on board device";
		$last_file="$path/on_board_device.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (/^Memory Device/) {
		$start_tag=1;
		$last_tag="memory device";
		$last_file="$path/memory_device.xml";
		output($last_file, '') if !-e $last_file;
	    }

	    if (($last_tag eq "bios" || $last_tag eq "slot") && $start_tag == 1) { dmidecode("characteristic", $last_file, $_) }
	    if ($last_tag eq "processor" && $start_tag == 1) {
		if (/^.*Flags:/) { $startflag=1;$flagcpt=0 }
		$flag = 0 if /^\t/;
		if (/^\t\t/ && $startflag) { $flag=1;$flagcpt++ }
		if ($flagcpt > 0 && $flag == 0) { $endflag=1;$flagcpt=0 }
		else { $endflag = 0 }
		processor($flag, $endflag, $last_file, $_);
	    }

	    if ($last_tag eq "cache" && $start_tag == 1) { dmidecode("supported_sram_type", $last_file, $_) }
	    if (member($last_tag, "system", "base board", "chassis", "memory module", "port", "on board device", "memory device")
		  && $start_tag == 1) { 
		dmidecode("", $last_file, $_);
	    }

	}
    } else {

	exec("dmidecode");
	exit 1;
    }
}
				
sub lshw_collector {
    my ($path)=@_;
    my $last_tag;

    my $pid=open(my $PROC, "-|");
    if ($pid) {
	foreach (<$PROC>) {
			
	    if (/^.*\*-/ && $last_tag ne "") {
		if ($last_tag ne "processor") {
		    append_to_file(tag_to_file($path, $last_tag), "</$last_tag>\n");
		}
		$last_tag="";
	    }

	    my %re_to_tag = (
		'cpu' => 'processor',
		'cdrom' => 'removable',
		'disk' => 'storage',
		'network' => 'network',
		'display' => 'video_card',
		'communication' => 'modem',
		'multimedia' => 'sound_card',
	    );
	    my $re = join('|', keys %re_to_tag);

	    if (/^.*\*-($re)/) {	
		$last_tag = $re_to_tag{$1};
		append_to_file(tag_to_file($path, $last_tag), "<last_tag>\n");
	    }	
	    if ($last_tag) {
		my $f = $hclcollector::{"lshw_${last_tag}_line"};
		$f->($path, $_);
	    }
	}
    } else {
	exec("lshw");
	exit 1;
    }
}
							
sub cpuinfo {
    my ($path)=@_;
    my $procCount = 0;
    foreach (cat_('/proc/cpuinfo')) {
        $procCount++ if /^processor/;
    }
    append_to_file("$path/processor.xml", 
		   tagged_nl('processor_count', $procCount),
		   "</processor>\n");
}

sub dmidecode {
    my ($subtag, $file, $line)=@_;
    my @l;
    if ($line !~ m/^\t/) {
	$line =~ s/ /_/g;
	$line =~ s/\n$//g;
	$line = trim($line);
	$line = lc($line);
	push(@l, "<$line>\n");
	push(@pile, $line);
	push(@indent, "");
    }

    if ($line =~ m/^\t\t/) {
	$line =~ s/\n//g;
	$line = trim($line);	
	push @l, "\t\t" . tagged_nl($subtag, $line);
    }
    if ($line =~ m/^\t/) {
	if ($indent[scalar(@indent)-1] eq "\t") {
	    push(@l, pop(@indent) . "</" . pop(@pile) . ">\n");
	}
	
	$line = trim($line);
	$line =~ s/\n$//g;
	my @split = split(':', $line);
	$split[0] =~ s/ /_/g;
	$split[0] = lc($split[0]);
	$split[1] = trim($split[1]);
	if ($split[1] ne "") {
	    push @l, "\t" . tagged_nl($split[0], $split[1]);
	} else {
	    push(@l, "\t<$split[0]>\n");
	    push(@pile, $split[0]);
	    push(@indent, "\t");
	}
    }
    append_to_file($file, @l);
}

sub distribution {
    my ($path)=@_;
	
    foreach (cat_("/etc/mandriva-release")) {
	s/for/:/;
	my @split=split(":", $_);
	$split[0]=trim($split[0]);
	$split[1]=trim($split[1]);

	my @l;
	push(@l, "<linux_distribution>\n");
	push @l, "\t" . tagged_nl('distribution', $split[0]);
	push @l, "\t" . tagged_nl('architecture', $split[1]);
	push(@l, "</linux_distribution>\n");
	output("$path/distribution.xml", @l);
    }
}	

sub monitor {
    my ($path)=@_;
    my $start=0;
    my @l;

    foreach (cat_("/etc/X11/xorg.conf")) {
	$_=trim($_);
	if (/^Section "Monitor"/i) { $start=1 }
	if ($start == 1) {
	    if (!/^Section "Monitor"/i) {
		my @split = split(" ", $_);
		$split[1] =~ s/"//;
		push(@l, "\t" . tagged_nl('description', $split[1])) if /^.*Identifier/i;
		push(@l, "\t" . tagged_nl('vendor', $split[1])) if /^.*VendorName/i;
		push(@l, "\t" . tagged_nl('model', $split[1])) if /^.*ModelName/i;
		push(@l, "\t" . tagged_nl('horizontal_sync', $split[1])) if /^.*HorizSync/i;
		push(@l, "\t" . tagged_nl('vertical_refresh', $split[1])) if /^.*VertRefresh/i;
	    }
	}
	if (/^.*EndSection/i && $start == 1) { $start=0; push(@l, "</monitor>\n") }
    }
    output("$path/monitor.xml", @l);    
}

sub kernel {
    my ($path)=@_;
    my @l;
    push(@l, "<linux_kernel>\n");
    push @l, "\t" . tagged_nl('kernel', `uname -r`);
    push(@l, "</linux_kernel>\n");    
    output("$path/kernel.xml", @l);
}

sub lshw_processor_line {
    my ($path, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    if ($line !~ /^.*\*-cpu/ && $line !~ /^.*capabilities:/ && $line !~ /^.*description:/ && $line !~ /^.*physical id:/ && $line !~ /^.*bus info:/ && $line !~ /^.*slot:/ && $line !~ /^.*width:/ && $line !~ /^.*size:/) {
	push @l, tagged_nl($data[0], $data[1]);
    }
    append_to_file(tag_to_file($path, 'processor'), @l);
}


sub md5file {
    require  Digest::MD5;
    my ($path)=@_;
    output("$path/checksum", "");
    foreach my $file (glob_("$path/*")) {
	if ($file !~ /checksum/) {
	    open(my $FILE,$file);
	    binmode($FILE);
	    my $sum = Digest::MD5->new->addfile(*$FILE)->hexdigest;
	    append_to_file("$path/checksum", "$sum  " . basename($file) . "\n");
	}
    }
}

sub create_lspcidrake_file {
    my ($path)=@_;
    system("lspcidrake -v >$path/lspcidrake.out");
}

sub trim($) {
    my $string = shift;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;

    #substitution of special characters
	
    $string =~ s/\[//;
    $string =~ s/\]//;
    $string =~ s/\{//;
    $string =~ s/\}//;
    $string =~ s/\(//;
    $string =~ s/\)//;
    #$string =~ s/(\*|\+|\-|\.|\^|\$|\?|\\|\||"|'//;
    return $string;

}

sub processor {
    my ($flag, $eflag, $file, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    push @l, tagged_nl('family', $data[1]) if $line =~ /^.*Family:/;
    push(@l, "<flags>\n") if $line =~ /^.*Flags:/;

    if ($flag) {
	$line = trim($line);
	$line =~ s/\n$//;
	push(@l, "\t<flag>$line</flag>\n");
    }
    push(@l, "</flags>\n") if $eflag;
    push @l, tagged_nl('socket', $data[1]) if $line =~ /^.*Upgrade:/;
    append_to_file($file, @l);
}

sub lshw_removable_line {
    my ($path, $line) = @_;    
    my @data = lshw_parse_val($line);

    my @l;
    if (/^.*product:/) {
	push @l, product_string_to_vendor_device_ids($path, $data[1]);
    }

    if ($line !~ /^.*\*-cdrom/ && $line !~ /^.*physical id:/ && $line !~ /^.*logical name:/) {
	if ($line =~ /^.*capabilities:/) {
	    push @l, capabilities_to_xml($data[1]);
	} elsif ($line =~ /^.*bus info:/) {
	    my @split = split("@", $data[1]);
	    push @l, tagged_nl($data[0], $split[0]);
	} else {
	    push @l, tagged_nl($data[0], $data[1]);
	}
    }
    append_to_file(tag_to_file($path, 'removable'), @l);
}

sub lshw_storage_line {
    my ($path, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    if (/^.*product:/) {
	push @l, product_string_to_vendor_device_ids($path, $data[1]);
    }

    if ($line !~ /^.*\*-disk/ && $line !~ /^.*physical id:/ && $line !~ /^.*logical name:/) {
	if ($line =~ /^.*capabilities:/) {
	    push @l, capabilities_to_xml($data[1]);
	} elsif ($line =~ /^.*bus info:/) {
	    my @split = split("@", $data[1]);
	    push @l, tagged_nl($data[0], $split[0]);
	} else {
	    push @l, tagged_nl($data[0], $data[1]);
	}

    }
    append_to_file(tag_to_file($path, 'storage'), @l);
}

sub controller  {
    my ($path) = @_;
    my $start=0;
    my @l;
    my $pid=open(my $PROC, "-|");
    if ($pid) {
	foreach (<$PROC>) {
	    if ($start == 1) {
		my @data = split(': ', $_);
		$data[0] = trim($data[0]);
		$data[0] =~ s/ /_/;
		$data[1] = trim($data[1]);
		$data[1] =~ s/\n//;

		if (/^.*product:/) {
		    push @l, product_string_to_vendor_device_ids($path, $data[1]);
		}
					
		if (!/^.*physical id:/ && !/^.*logical name:/ && !/^.*\*-/) {
		    if (/^.*capabilities:/ || /^.*configuration:/) {
			if (/^.*capabilities:/) {
			    push @l, capabilities_to_xml($data[1]);
			} else {
			    my @split = split(" ", $data[1]);
			    my $find = 0;
			    while (scalar(@split) > 0 && $find == 0) {
				my $tmp = pop(@split);
				if ($tmp =~ /module/) {
				    my @split_ = split("=", $tmp);
				    push(@l, "<module>$split_[1]</module>\n");
				    $find = 1;
				}
			    }

			}

		    } elsif (/^.*bus info:/) {
			my @split = split("@", $data[1]);
			push @l, tagged_nl($data[0], $split[0]);
		    } else {
			push @l, tagged_nl($data[0], $data[1]);
		    }
		}
				
	    }

	    if (/^.*\*-/ && $start == 1) {
		if ($start == 1) {
		    push(@l, "</controller>\n");
		}
       		$start = 0;
	    }
	    if (/^.*\*-/ && !/^.*\*-core/) {
		$start = 1;
		push(@l, "<controller>\n");
	    }
	}
	push(@l, "</controller>\n");	
	output("$path/controller.xml", @l);
    } else {
	exec("lshw -class bus");
	exit 1;
    }	

}

sub lshw_network_line {
    my ($path, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    if (/^.*product:/) {
	push @l, product_string_to_vendor_device_ids($path, $data[1]);
    }

    if ($line !~ /^.*\*-network/ && $line !~ /^.*physical id:/) {
	if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
	    if ($line =~ /^.*capabilities:/) {
		push @l, capabilities_to_xml($data[1]);
            } else {
		my @split = split(" ", $data[1]);
		while (scalar(@split) > 0) {
		    my $tmp = pop(@split);
		    my @split_ = split("=", $tmp);
		    push @l, tagged_nl($split_[0], $split_[1]);
		}
	    }
        } elsif ($line =~ /^.*bus info:/) {
	    my @split = split("@", $data[1]);
            push @l, tagged_nl($data[0], $split[0]);
        } else {
            push @l, tagged_nl($data[0], $data[1]);
	}
    }
    append_to_file(tag_to_file($path, 'network'), @l);
}



sub lshw_video_card_line {
    my ($path, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    if (/^.*product:/) {
        product_string_to_vendor_device_ids($path, $data[1]);
    }

    if ($line !~ /^.*physical id:/ && $line !~ /^.*\*-display/) {
	if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
	    if ($line =~ /^.*capabilities:/) {
		push @l, capabilities_to_xml($data[1]);
            } else {
		my @split = split(" ", $data[1]);
		while (scalar(@split) > 0) {
		    my $tmp = pop(@split);
		    my @split_ = split("=", $tmp);
		    push @l, tagged_nl($split_[0], $split_[1]);
		}
	    }
        } elsif ($line =~ /^.*bus info:/) {
	    my @split = split("@", $data[1]);
            push @l, tagged_nl($data[0], $split[0]);
        } else {
            push @l, tagged_nl($data[0], $data[1]);
	}
    }
    append_to_file(tag_to_file($path, 'video_card'), @l);
}

sub input  {
    my ($path) = @_;
    my $start=0;
    my $pid=open(my $PROC, "-|");
    if ($pid) {
	my @l;
        foreach (<$PROC>) {
	    if ($start == 1) {
                my  @data = split(': ', $_);
		$data[0] = trim($data[0]);
		$data[0] =~ s/ /_/;
		$data[1] = trim($data[1]);
		$data[1] =~ s/\n//;

                if (/^.*product:/) {
		    push @l, product_string_to_vendor_device_ids($path, $data[1]);
		}
					
                if (!/^.*physical id:/ && !/^.*\*-/) {
		    if (/^.*capabilities:/ || /^.*configuration:/) {
                        if (/^.*capabilities:/) {
			    push @l, capabilities_to_xml($data[1]);
                        } else {
			    my @split = split(" ", $data[1]);
			    my $find = 0;
			    while (scalar(@split) > 0 && $find == 0) {
				my $tmp = pop(@split);
				if ($tmp =~ /module/) {
				    my @split_ = split("=", $tmp);
                                    push(@l, "<module>$split_[1]</module>\n");
				    $find = 1;
				}
			    }
			}
		    } elsif (/^.*bus info:/) {
			my @split = split("@", $data[1]);
                        push @l, tagged_nl($data[0], $split[0]);
		    } else {
                        push @l, tagged_nl($data[0], $data[1]);
		    }
		}
	    }

            if (/^.*\*-/ && $start == 1) {
		if ($start == 1) {
		    push(@l, "</input>\n");
		}
       		$start = 0;
	    }
            if (/^.*\*-/) {
		$start = 1;
                push(@l, "<input>\n");
	    }	
        }
        push(@l, "</input>\n");
	output("$path/input.xml", @l);
    } else {
        exec("lshw -class input");
        exit 1;
    }	

}



sub lshw_modem_line {
    my ($path, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    if (/^.*product:/) {
        product_string_to_vendor_device_ids($path, $data[1]);
    }
					
    if ($line !~ /^.*physical id:/ && $line !~ /^.*\*-/) {
	if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
	    if ($line =~ /^.*capabilities:/) {
		push @l, capabilities_to_xml($data[1]);
            } else {
		my @split = split(" ", $data[1]);
		my $find = 0;
		while (scalar(@split) > 0 && $find == 0) {
		    my $tmp = pop(@split);
		    if ($tmp =~ /module/) {
			my @split_ = split("=", $tmp);
                        push(@l, "<module>$split_[1]</module>\n");
			$find = 1;
		    }
		}

	    }

        } elsif ($line =~ /^.*bus info:/) {
	    my @split = split("@", $data[1]);
            push @l, tagged_nl($data[0], $split[0]);
        } else {
            push @l, tagged_nl($data[0], $data[1]);
	}
    }	
    append_to_file(tag_to_file($path, 'modem'), @l);
}

sub bridge  {
    my ($path) = @_;
    my $start=0;
    my $pid=open(my $PROC, "-|");
    if ($pid) {
	my @l;
        foreach (<$PROC>) {

	    if ($start == 1) {
                my @data = split(': ', $_);
		$data[0] = trim($data[0]);
		$data[0] =~ s/ /_/;
		$data[1] = trim($data[1]);
		$data[1] =~ s/\n//;

                if (/^.*product:/) {
		    product_string_to_vendor_device_ids($path, $data[1]);		    
                }
					
                if (!/^.*physical id:/ && !/^.*logical name:/ && !/^.*\*-/) {
		    if (/^.*capabilities:/ || /^.*configuration:/) {
                        if (/^.*capabilities:/) {
			    push @l, capabilities_to_xml($data[1]);
                        } else {
			    my @split = split(" ", $data[1]);
			    my $find = 0;
			    while (scalar(@split) > 0 && $find == 0) {
				my $tmp = pop(@split);
				if ($tmp =~ /module/) {
				    my @split_ = split("=", $tmp);
                                    push(@l, "<module>$split_[1]</module>\n");
				    $find = 1;
				}
			    }
			}
		    } elsif (/^.*bus info:/) {
			my @split = split("@", $data[1]);
                        push @l, tagged_nl($data[0], $split[0]);
		    } else {
                        push @l, tagged_nl($data[0], $data[1]);
		    }
		}
	    }
            if (/^.*\*-/ && $start == 1) {
		if ($start == 1) {
		    push(@l, "</bridge>\n");
		}
       		$start = 0;
	    }
            if (/^.*\*-/) {
		$start = 1;
                push(@l, "<bridge>\n");
            }
        }
        push(@l, "</bridge>\n");
	output("$path/bridge.xml", @l);
    } else {
        exec("lshw -class bridge");
        exit 1;
    }	

}

sub lshw_sound_card_line {
    my ($path, $line) = @_;
    my @data = lshw_parse_val($line);

    my @l;
    if (/^.*product:/) {
        product_string_to_vendor_device_ids($path, $data[1]);
    }

    if ($line !~ /^.*multimedia/ && $line !~ /^.*physical id:/) {
	if ($line =~ /^.*capabilities:/ || $line =~ /^.*configuration:/) {
	    if ($line =~ /^.*capabilities:/) {
		push @l, capabilities_to_xml($data[1]);
            } else {
		my @split = split(" ", $data[1]);
		while (scalar(@split) > 0) {
		    my $tmp = pop(@split);
		    my @split_ = split("=", $tmp);
		    push @l, tagged_nl($split_[0], $split_[1]);
		}
	    }
        } elsif ($line =~ /^.*bus info:/) {
	    my @split = split("@", $data[1]);
            push @l, tagged_nl($data[0], $split[0]);
        } else {
            push @l, tagged_nl($data[0], $data[1]);
	}
    }
    append_to_file(tag_to_file($path, 'sound_card'), @l);
}

sub product_string_to_vendor_device_ids {
    my ($path, $product) = @_;

    $product = trim($product);
    if (my ($info) = grep { trim($_) =~ /^.*\|$product/ } cat_("$path/lspcidrake.out")) {
	my ($ids) = $info =~ /\((vendor:.*)\)/;
	map {
	    my ($name, $val) = split(":", $_);
	    tagged_nl($name . '_id', $val);
	} split(" ", $ids);
    } else { () }
}

sub lshw_parse_val {
    my ($line) = @_;
    my @data = split(': ', $line);
    $data[0] = trim($data[0]);
    $data[0] =~ s/ /_/;
    $data[1] = trim($data[1]);
    $data[1] =~ s/\n//;
    @data;
}

sub capabilities_to_xml {
    my ($capabilities) = @_;

    my @l;
    push(@l, "<capabilities>\n");
    push @l, map {
	"\t" . tagged_nl('capability', $_);
    } reverse split(" ", $capabilities);
    push(@l, "</capabilities>\n");
    @l;
}

sub tagged_nl {
    my ($tag, $val) = @_;
    tagged($tag, $val) . "\n";
}
sub tagged {
    my ($tag, $val) = @_;
    "<$tag>$val</$tag>";
}

sub tag_to_file {
    my ($path, $tag) = @_;
    "$path/$tag.xml";
}

1;

